package jp.hishidama.win32.shdocvw;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;

import jp.hishidama.win32.api.OcIdlConst;
import jp.hishidama.win32.com.ComPtr;
import jp.hishidama.win32.com.IDispatch;
import jp.hishidama.win32.mshtml.IHTMLDocument;

/**
 * IWebBrowser,IWebBrowser2,IWebBrowserAppNX.
 * 
 * @author <a target="hishidama"
 *         href="http://www.ne.jp/asahi/hishidama/home/soft/java/ierobot.html">Ђ</a>
 * @since 2007.10.22
 */
public class IWebBrowser extends IDispatch {

	/**
	 * VKWebBrowser쐬.
	 * 
	 * @return EFuuEU[
	 */
	public static IWebBrowser create() {
		long ptr = Native.create();
		if (ptr == 0)
			return null;
		return new IWebBrowser(ptr);
	}

	/**
	 * WebBrowserT.
	 * 
	 * @param hwnd
	 *            HWND
	 * @return EFuuEU[
	 */
	public static IWebBrowser findWebBrowser(long hwnd) {
		if (hwnd == 0) {
			return null;
		}

		long ptr = Native.findWebBrowser(hwnd);
		if (ptr == 0)
			return null;
		return new IWebBrowser(ptr);
	}

	/**
	 * WebBrowser.
	 * 
	 * @return {@link IWebBrowser}̃Xg
	 */
	public static List enumWebBrowser() {
		List list = new ArrayList();
		Native.enumWebBrowser(list);
		return list;
	}

	protected IWebBrowser(long dll_ptr) {
		super(dll_ptr);
	}

	/**
	 * ̑Oy[Wֈړ.
	 */
	public void goBack() {
		Native.GoBack(getPtr());
	}

	/**
	 * ̎y[Wֈړ.
	 */
	public void goForward() {
		Native.GoForward(getPtr());
	}

	/**
	 * z[ֈړ.
	 */
	public void goHome() {
		Native.GoHome(getPtr());
	}

	/**
	 * y[Wֈړ.
	 */
	public void goSearch() {
		Native.GoSearch(getPtr());
	}

	/**
	 * wURLֈړ.
	 * 
	 * @param url
	 *            URL
	 */
	public void navigate(String url) {
		Native.Navigate(getPtr(), url, null, null);
	}

	/**
	 * wURLֈړ.
	 * 
	 * @param url
	 *            URL
	 * @param flags
	 *            tO
	 * @param targetFrameName
	 *            t[
	 */
	public void navigate(String url, String flags, String targetFrameName) {
		Native.Navigate(getPtr(), url, flags, targetFrameName);
	}

	/**
	 * XV.
	 */
	public void refresh() {
		Native.Refresh(getPtr());
	}

	/**
	 * ~.
	 */
	public void stop() {
		Native.Stop(getPtr());
	}

	/**
	 * HTMLhLg擾.
	 * 
	 * @param addChild
	 *            trueFAj
	 * @return HTMLhLg
	 * @see ComPtr#disposeChild()
	 */
	public IHTMLDocument getDocument(boolean addChild) {
		IHTMLDocument doc = (IHTMLDocument) Native.get_Document(getPtr());
		if (addChild) {
			addChildComPtr(doc);
		}
		return doc;
	}

	/**
	 * hLgIuWFNg擾.
	 * 
	 * @return 
	 */
	public String getType() {
		return Native.get_Type(getPtr());
	}

	/**
	 * ʒu擾.
	 * 
	 * @return ʒu
	 */
	public Point getLocation() {
		long ptr = getPtr();
		int x = Native.get_Left(ptr);
		int y = Native.get_Top(ptr);
		return new Point(x, y);
	}

	/**
	 * ʒuݒ.
	 * 
	 * @param x
	 *            X
	 * @param y
	 *            Y
	 */
	public void setLocation(int x, int y) {
		long ptr = getPtr();
		Native.put_Left(ptr, x);
		Native.put_Top(ptr, y);
	}

	/**
	 * TCY擾.
	 * 
	 * @return TCY
	 */
	public Dimension getSize() {
		long ptr = getPtr();
		int cx = Native.get_Width(ptr);
		int cy = Native.get_Height(ptr);
		return new Dimension(cx, cy);
	}

	/**
	 * TCYݒ.
	 * 
	 * @param cx
	 *            
	 * @param cy
	 *            
	 */
	public void setSize(int cx, int cy) {
		long ptr = getPtr();
		Native.put_Width(ptr, cx);
		Native.put_Height(ptr, cy);
	}

	/**
	 * ʒuTCY擾.
	 * 
	 * @return `
	 */
	public Rectangle getBounds() {
		long ptr = getPtr();
		int x = Native.get_Left(ptr);
		int y = Native.get_Top(ptr);
		int cx = Native.get_Width(ptr);
		int cy = Native.get_Height(ptr);
		return new Rectangle(x, y, cx, cy);
	}

	/**
	 * ʒuTCYݒ.
	 * 
	 * @param x
	 *            X
	 * @param y
	 *            Y
	 * @param cx
	 *            
	 * @param cy
	 *            
	 */
	public void setBounds(int x, int y, int cx, int cy) {
		long ptr = getPtr();
		Native.put_Left(ptr, x);
		Native.put_Top(ptr, y);
		Native.put_Width(ptr, cx);
		Native.put_Height(ptr, cy);
	}

	/**
	 * XW擾.
	 * 
	 * @return X
	 */
	public int getLeft() {
		return Native.get_Left(getPtr());
	}

	/**
	 * XWݒ.
	 * 
	 * @param x
	 *            X
	 */
	public void setLeft(int x) {
		Native.put_Left(getPtr(), x);
	}

	/**
	 * YW擾.
	 * 
	 * @return Y
	 */
	public int getTop() {
		return Native.get_Top(getPtr());
	}

	/**
	 * YWݒ.
	 * 
	 * @param y
	 *            Y
	 */
	public void setTop(int y) {
		Native.put_Top(getPtr(), y);
	}

	/**
	 * 擾.
	 * 
	 * @return 
	 */
	public int getWidth() {
		return Native.get_Width(getPtr());
	}

	/**
	 * ݒ.
	 * 
	 * @param width
	 *            
	 */
	public void setWidth(int width) {
		Native.put_Width(getPtr(), width);
	}

	/**
	 * 擾.
	 * 
	 * @return 
	 */
	public int getHeight() {
		return Native.get_Height(getPtr());
	}

	/**
	 * ݒ.
	 * 
	 * @param height
	 *            
	 */
	public void setHeight(int height) {
		Native.put_Height(getPtr(), height);
	}

	/**
	 * P[V擾.
	 * 
	 * @return 
	 */
	public String getLocationName() {
		return Native.get_LocationName(getPtr());
	}

	/**
	 * P[VURL擾.
	 * 
	 * @return URL
	 */
	public String getLocationURL() {
		return Native.get_LocationURL(getPtr());
	}

	/**
	 * rW[Ԏ擾.
	 * 
	 * @return rW[
	 */
	public boolean getBusy() {
		return Native.get_Busy(getPtr());
	}

	/**
	 * I.
	 */
	public void quit() {
		Native.Quit(getPtr());
	}

	/**
	 * Rg[̖̎擾.
	 * 
	 * @return 
	 */
	public String getName() {
		return Native.get_Name(getPtr());
	}

	/**
	 * HWND擾.
	 * 
	 * @return HWND
	 */
	public long getHWND() {
		return Native.get_HWND(getPtr());
	}

	/**
	 * sW[̃tpX擾.
	 * 
	 * @return tpX
	 */
	public String getFullName() {
		return Native.get_FullName(getPtr());
	}

	/**
	 * sW[̃pX擾.
	 * 
	 * @return pX
	 */
	public String getPath() {
		return Native.get_Path(getPtr());
	}

	/**
	 * \Ԏ擾.
	 * 
	 * @return \
	 */
	public boolean getVisible() {
		return Native.get_Visible(getPtr());
	}

	/**
	 * \Ԑݒ.
	 * 
	 * @param b
	 *            \
	 */
	public void setVisible(boolean b) {
		Native.put_Visible(getPtr(), b);
	}

	/**
	 * Xe[^Xo[\Ԏ擾.
	 * 
	 * @return \
	 */
	public boolean getStatusBar() {
		return Native.get_StatusBar(getPtr());
	}

	/**
	 * Xe[^Xo[\Ԑݒ.
	 * 
	 * @param b
	 *            \
	 */
	public void setStatusBar(boolean b) {
		Native.put_StatusBar(getPtr(), b);
	}

	/**
	 * Xe[^Xo[\擾.
	 * 
	 * @return 
	 */
	public String getStatusText() {
		return Native.get_StatusText(getPtr());
	}

	/**
	 * Xe[^Xo[\.
	 * 
	 * @param text
	 *            
	 */
	public void setStatusText(String text) {
		Native.put_StatusText(getPtr(), text);
	}

	/**
	 * c[o[\Ԏ擾.
	 * 
	 * @return \
	 */
	public int getToolBar() {
		return Native.get_ToolBar(getPtr());
	}

	/**
	 * c[o[\Ԑݒ.
	 * 
	 * @param value
	 *            \
	 */
	public void setToolBar(int value) {
		Native.put_ToolBar(getPtr(), value);
	}

	/**
	 * j[o[\Ԏ擾.
	 * 
	 * @return \
	 */
	public boolean getMenuBar() {
		return Native.get_MenuBar(getPtr());
	}

	/**
	 * j[o[\Ԑݒ.
	 * 
	 * @param b
	 *            \
	 */
	public void setMenuBar(boolean b) {
		Native.put_MenuBar(getPtr(), b);
	}

	/**
	 * tXN[Ԏ擾.
	 * 
	 * @return tXN[
	 */
	public boolean getFullScreen() {
		return Native.get_FullScreen(getPtr());
	}

	/**
	 * tXN[Ԑݒ.
	 * 
	 * @param b
	 *            tXN[
	 */
	public void setFullScreen(boolean b) {
		Native.put_FullScreen(getPtr(), b);
	}

	/**
	 * ReadyState擾.
	 * 
	 * @return ReadyState
	 * @see OcIdlConst#READYSTATE_COMPLETE
	 */
	public int getReadyState() {
		return Native.get_ReadyState(getPtr());
	}

	/**
	 * ItCԎ擾.
	 * 
	 * @return ItC
	 */
	public boolean getOffline() {
		return Native.get_Offline(getPtr());
	}

	/**
	 * ItCԐݒ.
	 * 
	 * @param b
	 *            ItC
	 */
	public void setOffline(boolean b) {
		Native.put_Offline(getPtr(), b);
	}

	/**
	 * _CAO\ێ擾.
	 * 
	 * @return \
	 */
	public boolean getSilent() {
		return Native.get_Silent(getPtr());
	}

	/**
	 * _CAO\ېݒ.
	 * 
	 * @param b
	 *            \
	 */
	public void setSilent(boolean b) {
		Native.put_Silent(getPtr(), b);
	}

	/**
	 * gbvxuEU[o^Ԏ擾.
	 * 
	 * @return o^
	 */
	public boolean getRegisterAsBrowser() {
		return Native.get_RegisterAsBrowser(getPtr());
	}

	/**
	 * gbvxuEU[o^Ԑݒ.
	 * 
	 * @param b
	 *            o^
	 */
	public void setRegisterAsBrowser(boolean b) {
		Native.put_RegisterAsBrowser(getPtr(), b);
	}

	/**
	 * hbv^[Qbgo^Ԏ擾.
	 * 
	 * @return o^
	 */
	public boolean getRegisterAsDropTarget() {
		return Native.get_RegisterAsDropTarget(getPtr());
	}

	/**
	 * hbv^[Qbgo^Ԑݒ.
	 * 
	 * @param b
	 *            o^
	 */
	public void setRegisterAsDropTarget(boolean b) {
		Native.put_RegisterAsDropTarget(getPtr(), b);
	}

	/**
	 * VA^[Ԏ擾.
	 * 
	 * @return VA^[
	 */
	public boolean getTheaterMode() {
		return Native.get_TheaterMode(getPtr());
	}

	/**
	 * VA^[Ԑݒ.
	 * 
	 * @param b
	 *            VA^[
	 */
	public void setTheaterMode(boolean b) {
		Native.put_TheaterMode(getPtr(), b);
	}

	/**
	 * AhXo[\Ԏ擾.
	 * 
	 * @return \
	 */
	public boolean getAddressBar() {
		return Native.get_AddressBar(getPtr());
	}

	/**
	 * AhXo[\Ԑݒ.
	 * 
	 * @param b
	 *            \
	 */
	public void setAddressBar(boolean b) {
		Native.put_AddressBar(getPtr(), b);
	}

	/**
	 * TCYύXێ擾.
	 * 
	 * @return ύX
	 */
	public boolean getResizable() {
		return Native.get_Resizable(getPtr());
	}

	/**
	 * TCYύXېݒ.
	 * 
	 * @param b
	 *            ύX
	 */
	public void setResizable(boolean b) {
		Native.put_Resizable(getPtr(), b);
	}

	private static class Native {
		//
		// Original
		//

		private static native long create();

		private static native void enumWebBrowser(List list);

		private static native long findWebBrowser(long hwnd);

		//
		// IWebBrowser
		//

		private static native void GoBack(long ptr);

		private static native void GoForward(long ptr);

		private static native void GoHome(long ptr);

		private static native void GoSearch(long ptr);

		private static native void Navigate(long ptr, String url, String flags,
				String targetFrameName);

		private static native void Refresh(long ptr);

		private static native void Stop(long ptr);

		// get_Application
		// get_Parent

		private static native IDispatch get_Document(long ptr);

		private static native String get_Type(long ptr);

		private static native int get_Left(long ptr);

		private static native void put_Left(long ptr, int left);

		private static native int get_Top(long ptr);

		private static native void put_Top(long ptr, int top);

		private static native int get_Width(long ptr);

		private static native void put_Width(long ptr, int width);

		private static native int get_Height(long ptr);

		private static native void put_Height(long ptr, int height);

		private static native String get_LocationName(long ptr);

		private static native String get_LocationURL(long ptr);

		private static native boolean get_Busy(long ptr);

		//
		// IWebBrowserApp
		//

		private static native void Quit(long ptr);

		private static native String get_Name(long ptr);

		private static native long get_HWND(long ptr);

		private static native String get_FullName(long ptr);

		private static native String get_Path(long ptr);

		private static native boolean get_Visible(long ptr);

		private static native void put_Visible(long ptr, boolean b);

		private static native boolean get_StatusBar(long ptr);

		private static native void put_StatusBar(long ptr, boolean b);

		private static native String get_StatusText(long ptr);

		private static native void put_StatusText(long ptr, String text);

		private static native int get_ToolBar(long ptr);

		private static native void put_ToolBar(long ptr, int value);

		private static native boolean get_MenuBar(long ptr);

		private static native void put_MenuBar(long ptr, boolean b);

		private static native boolean get_FullScreen(long ptr);

		private static native void put_FullScreen(long ptr, boolean b);

		//
		// IWebBrowser2
		//

		private static native int get_ReadyState(long ptr);

		private static native boolean get_Offline(long ptr);

		private static native void put_Offline(long ptr, boolean b);

		private static native boolean get_Silent(long ptr);

		private static native void put_Silent(long ptr, boolean b);

		private static native boolean get_RegisterAsBrowser(long ptr);

		private static native void put_RegisterAsBrowser(long ptr, boolean b);

		private static native boolean get_RegisterAsDropTarget(long ptr);

		private static native void put_RegisterAsDropTarget(long ptr, boolean b);

		private static native boolean get_TheaterMode(long ptr);

		private static native void put_TheaterMode(long ptr, boolean b);

		private static native boolean get_AddressBar(long ptr);

		private static native void put_AddressBar(long ptr, boolean b);

		private static native boolean get_Resizable(long ptr);

		private static native void put_Resizable(long ptr, boolean b);
	}
}
